import os

import pytest
from click.testing import CliRunner
from pytest_lazyfixture import lazy_fixture
from pyfakefs.fake_filesystem import FakeFilesystem

from gcs_migration.commands.gcs_migration import main
from gcs_migration.models import MigrationPlan


def test_snapshot_list(
    migration_plan: MigrationPlan,
    fs: FakeFilesystem,
) -> None:
    descriptions = ["a", None, "A much longer description"]
    offset = 190
    migration_plan.dump()

    runner = CliRunner()

    os.mkdir("migration_snapshots")
    with open(f"migration_snapshots/{offset:04}", "w"):
        pass

    for description in descriptions:
        if description:
            args = ["snapshot", "create", "-d", description]
        else:
            args = ["snapshot", "create"]
        result = runner.invoke(main, args)
        assert result.exit_code == 0

    result = runner.invoke(main, ["snapshot", "list"])
    assert result.exit_code == 0

    for line in result.output.split("\n")[2:-1]:
        snapshot_id_str, timestamp, description = line.split(" | ")
        description_id = int(snapshot_id_str.strip()) - (offset + 1)
        assert description_id < len(descriptions)
        if descriptions[description_id]:
            assert descriptions[description_id] == description.strip()
        else:
            assert description.strip() == ""


@pytest.mark.parametrize("omit_secrets", [False, True])
@pytest.mark.parametrize(
    "migration_plan_with_secrets",
    [
        lazy_fixture("migration_plan_s3"),
        lazy_fixture("migration_plan_blackpearl"),
        lazy_fixture("migration_plan_ceph"),
    ],
)
def test_snapshot_list_without_secrets(
    migration_plan_with_secrets: MigrationPlan,
    fs: FakeFilesystem,
    omit_secrets: bool,
) -> None:
    descriptions = ["a", None, "A much longer description"]
    offset = 190
    migration_plan_with_secrets.dump()

    runner = CliRunner()

    os.mkdir("migration_snapshots")
    with open(f"migration_snapshots/{offset:04}", "w"):
        pass

    for description in descriptions:
        if description:
            args = ["snapshot", "create", "-d", description]
        else:
            args = ["snapshot", "create"]
            if omit_secrets:
                args.append("-o")
        result = runner.invoke(main, args)
        assert result.exit_code == 0

    result = runner.invoke(main, ["snapshot", "list"])
    assert result.exit_code == 0

    for line in result.output.split("\n")[2:-2]:
        snapshot_id_str, timestamp, description = line.split(" | ")
        description_id = int(snapshot_id_str.strip()) - (offset + 1)
        assert description_id < len(descriptions)
        if descriptions[description_id]:
            assert descriptions[description_id] == description.strip()
        else:
            if omit_secrets:
                assert description.strip() == "*"
            else:
                assert description.strip() == ""
    if omit_secrets:
        assert "Globus support" in result.output.split("\n")[-2]
