import os
from typing import Optional

import pytest
from click.testing import CliRunner
from pyfakefs.fake_filesystem import FakeFilesystem

from gcs_migration.commands.gcs_migration import main
from gcs_migration.models import MigrationPlan


@pytest.mark.parametrize(
    "snapshot_number,description,expect",
    [(None, "Snapshot description", 0), (1, None, 1), (9, None, 1), (10, None, 0)],
)
def test_snapshot_restore(
    migration_plan: MigrationPlan,
    fs: FakeFilesystem,
    snapshot_number: Optional[int],
    description: Optional[str],
    expect: int,
) -> None:
    migration_plan.dump()

    runner = CliRunner()
    args = ["snapshot", "create"]

    if description:
        args.extend(["-d", description])

    os.mkdir("migration_snapshots")
    with open("migration_snapshots/0009", "w"):
        pass

    result = runner.invoke(main, args)
    assert result.exit_code == 0

    args = ["snapshot", "restore"]
    if snapshot_number is not None:
        args.append(str(snapshot_number))

    result = runner.invoke(main, args, catch_exceptions=False)
    assert result.exit_code == expect

    assert os.path.exists("migration_plan")

    if expect == 0:
        restored_migration_plan = MigrationPlan.load()
        assert restored_migration_plan == migration_plan
        assert "Restored to" in result.output
        if description:
            assert description in result.output


def test_snapshot_restore_none(fs: FakeFilesystem) -> None:
    runner = CliRunner()
    args = ["snapshot", "restore"]

    result = runner.invoke(main, args, catch_exceptions=False)
    assert result.exit_code == 1
