import pytest

from gcs_migration.models import (
    BlackPearlCredentialYamlDict,
    CephCredentialYamlDict,
    CredentialYamlDict,
    HPSSCredentialYamlDict,
    S3CredentialYamlDict,
)


@pytest.fixture
def credential(
    test_user_cert_subject: str, sharing_local_user: str
) -> CredentialYamlDict:
    return {
        "username": sharing_local_user,
        "subject": test_user_cert_subject,
        "gsi_authz_callout_index": 0,
        "identity_id": "f8ba8dee-a4d2-46eb-86f1-4157522872a8",
        "identity_username": f"{sharing_local_user}@globus.org",
    }


@pytest.fixture
def s3_credential(
    test_user_cert_subject: str, sharing_local_user: str
) -> S3CredentialYamlDict:
    return {
        "username": f"{sharing_local_user}@globus.org",
        "subject": test_user_cert_subject,
        "gsi_authz_callout_index": 0,
        "identity_id": "f8ba8dee-a4d2-46eb-86f1-4157522872a8",
        "identity_username": f"{sharing_local_user}@globus.org",
        "s3_key_id": None,
        "s3_secret_key": None,
    }


@pytest.fixture
def ceph_credential(
    test_user_cert_subject: str, sharing_local_user: str
) -> CephCredentialYamlDict:
    return {
        "username": f"{sharing_local_user}",
        "subject": test_user_cert_subject,
        "gsi_authz_callout_index": 0,
        "identity_id": "f8ba8dee-a4d2-46eb-86f1-4157522872a8",
        "identity_username": f"{sharing_local_user}@globus.org",
    }


@pytest.fixture
def blackpearl_credential(
    test_user_cert_subject: str, sharing_local_user: str
) -> BlackPearlCredentialYamlDict:
    return {
        "username": f"{sharing_local_user}",
        "subject": test_user_cert_subject,
        "gsi_authz_callout_index": 0,
        "identity_id": "f8ba8dee-a4d2-46eb-86f1-4157522872a8",
        "identity_username": f"{sharing_local_user}@globus.org",
        "access_id": None,
        "secret_key": None,
    }


@pytest.fixture
def hpss_credential(
    test_user_cert_subject: str, sharing_local_user: str
) -> HPSSCredentialYamlDict:
    return {
        "username": f"{sharing_local_user}",
        "subject": test_user_cert_subject,
        "gsi_authz_callout_index": 0,
        "identity_id": "f8ba8dee-a4d2-46eb-86f1-4157522872a8",
        "identity_username": f"{sharing_local_user}@globus.org",
    }


@pytest.fixture
def credential_unresolved(
    test_user_cert_subject: str, sharing_local_user: str
) -> CredentialYamlDict:
    return {
        "username": sharing_local_user,
        "subject": test_user_cert_subject,
        "gsi_authz_callout_index": 0,
        "identity_id": None,
        "identity_username": None,
    }
