import os
from typing import Callable, Dict, Iterator, Optional

import pytest
from pyfakefs.fake_filesystem import FakeFilesystem

from gcs_migration.parsers import HPSSConfig


@pytest.fixture
def hpss_config_file_default_path() -> str:
    return "/var/hpss/etc/gridftp_hpss_dsi.conf"


@pytest.fixture
def hpss_config_default_login_name() -> str:
    return "hpssftp"


@pytest.fixture
def hpss_config_default_uda_checksum() -> str:
    return "off"


@pytest.fixture
def hpss_login_name(hpss_config_default_login_name: str) -> str:
    return hpss_config_default_login_name


@pytest.fixture
def hpss_authenticator(hpss_write_file) -> str:
    path = "/bar"
    hpss_write_file(path, {}, "")
    return f"auth_keyfile:{path}"


@pytest.fixture
def hpss_authentication_mech() -> str:
    return "krb5"


@pytest.fixture
def hpss_uda_checksum_support() -> str:
    return "on"


@pytest.fixture
def hpss_config_file_default_contents(
    hpss_login_name: str,
    hpss_authenticator: str,
    hpss_authentication_mech: str,
    hpss_uda_checksum_support: str,
) -> Dict[str, str]:
    return {
        "LoginName": hpss_login_name,
        "Authenticator": hpss_authenticator,
        "AuthenticationMech": hpss_authentication_mech,
        "UDAChecksumSupport": hpss_uda_checksum_support,
    }


@pytest.fixture
def hpss_env_file_default_contents() -> Dict[str, str]:
    return {
        "HPSS_API_AUTHN_MECH": "foo",
        "HPSS_PRIMARY_AUTHN_MECH": "bar",
        "HPSS_PRIMARY_AUTHENTICATOR": "auth_keytab:/foo",
    }


@pytest.fixture
def hpss_env_file_default_path() -> str:
    return "/var/hpss/etc/env.conf"


@pytest.fixture
def hpss_write_file(
    fs: FakeFilesystem,
) -> Iterator[Callable[[str, Dict[str, Optional[str]], str], None]]:
    def _write_file(
        path: str, config: Dict[str, Optional[str]], separator: str
    ) -> None:
        if not os.path.exists(os.path.dirname(path)):
            fs.makedirs(os.path.dirname(path))
        with open(path, "w") as f:
            for k, v in config.items():
                f.write(f"{k}{separator}{v if v is not None else ''}\n")

    yield _write_file


@pytest.fixture
def hpss_write_config_file(
    hpss_write_file,
) -> Callable[[str, Dict[str, Optional[str]]], None]:
    return lambda path, config: hpss_write_file(path, config, " ")


@pytest.fixture
def hpss_write_env_file(
    hpss_write_file,
) -> Callable[[str, Dict[str, Optional[str]]], None]:
    return lambda path, config: hpss_write_file(path, config, "=")


@pytest.fixture
def hpss_config_file(
    hpss_write_config_file,
    hpss_config_file_default_path,
    hpss_config_file_default_contents,
) -> str:
    hpss_write_config_file(
        hpss_config_file_default_path, hpss_config_file_default_contents
    )
    return hpss_config_file_default_path


@pytest.fixture
def hpss_config(hpss_config_file: str) -> HPSSConfig:
    return HPSSConfig(hpss_config_file)
